#!/bin/bash
################################################################################################
#Scrypt for creating mail-server statistic                                                   	#
#Need wkhtmltopdf, gnuplot, base64 and mutt							#
#												#
# If you have no X-server, than you need static-build version of wkhtmltopdf			#
# https://code.google.com/p/wkhtmltopdf/downloads/list						#
#												#
#This scrypt was tested on postfix whith spamassassin						#
#												#
#Created by Serge Yakimchuck									#
#mail to: yakim@yakim.org.ua									#
#												#
################################################################################################
addr=admin@your.domain #address to send statistic
HTML2PDF="/usr/bin/wkhtmltopdf"
GNUPLOT="/usr/bin/gnuplot"
ST_FILE=stat.html
log=/var/log/mail.log
old_log=/var/log/mail.log.1
png_stat=mail.png
MAILFROM="statistic@your.domain"
SUBJECT="MailServer Statistic"
###################################
# Messages in html and pdf        #
###################################
gp_mail_count="Количество писем" # Numbers of messages
gp_date="Дата" #Date
gp_rejected="Не принятые" # Rejected
gp_spam="Спам" #Spam
gp_normal="Нормальные" #Normal messages
html_mail_stat="Статистика почтового сервера" #HTML mail statistic
html_all_messages="Всего писем" #Total messages
html_rejected="Отвергнуто писем" #Rejected messages
html_spam="Опознано как спам"	# Recognized as spam
html_normal="Нормальных писем"	#Normal messages
html_histogram_name="Статистика почтового сервера за" #Server statistic for
html_histogram_day="день" #day
html_histogram_7days="7 дней" #7 days





tmp='/tmp/stats'
max_mail=0

mkdir $tmp

function create_gnuplot {
let "max_mail = max_mail+200"
echo "#! /usr/bin/gnuplot -persist" > $tmp/out.gp
echo "set terminal png size 800, 600" >> $tmp/out.gp
echo "set output \"$tmp/mail.png\" " >> $tmp/out.gp
echo "set encoding koi8r " >> $tmp/out.gp
echo "set xlabel \"$gp_date\" font \"Helvetica,18\" " >> $tmp/out.gp
echo "set style data histograms " >> $tmp/out.gp
echo "set bmargin 4 " >> $tmp/out.gp
echo "set ylabel \"$gp_mail_count\" font \"Helvetica,18\" " >> $tmp/out.gp
echo "set style fill solid 1.00 border -1 " >> $tmp/out.gp
echo "set grid ytics " >> $tmp/out.gp
echo "set boxwidth 0.75 " >> $tmp/out.gp
echo "set style data histograms " >> $tmp/out.gp
echo "set style histogram rowstacked " >> $tmp/out.gp
echo "set yrange [0:$max_mail] " >> $tmp/out.gp

echo "plot \"$tmp/mail.dat\" using 2:xtic(1) ti \"$gp_rejected\", '' u 3 ti \"$gp_spam\", '' u 4 ti \"$gp_normal\",\\" >> $tmp/out.gp
echo "'' using 0:2:2 with labels font \"Liberation Serif Bold, 12\" offset 0,-1 notitle,\\" >> $tmp/out.gp
echo "'' using 0:(\$3+\$2-5):3 with labels font \"Liberation Serif Bold, 12\" offset 0,-0 notitle,\\" >> $tmp/out.gp
echo "'' using 0:(\$4+\$3+\$2):4 with labels font \"Liberation Serif Bold, 12\" offset 0,-2 notitle" >> $tmp/out.gp
$GNUPLOT $tmp/out.gp
}


function create_html () {
local dt=0
let "rej_proc = reject_mail *100 / all_mail"
let "spam_proc = spamassassin_mail *100 / all_mail"
let "sent_proc = sent_mail *100 / all_mail"
echo "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"/>"> $tmp/$ST_FILE
echo "<html><body><h1 $html_mail_stat /h1>" >> $tmp/$ST_FILE
if [ "$1" = 'week' ]; 
then
dt="$html_histogram_7days"
else
#dt="$html_histogram_day — "`LC_ALL=C date --date="yesterday" +%b' '%e`
dt="$html_histogram_day — "`date --date="yesterday" +%b' '%e`
fi

echo "<font size="3"><p>$html_histogram_name $dt</p>" >> $tmp/$ST_FILE
echo "<IMG  SRC=\"data:image/png;base64," >> $tmp/$ST_FILE
/usr/bin/base64 $tmp/$png_stat >> $tmp/$ST_FILE
echo "\"alt=\"mail statistic\">" >> $tmp/$ST_FILE
echo "<font size="3"><p>$html_all_messages $all_mail — 100%</p>" >> $tmp/$ST_FILE
echo "<p>$html_rejected $reject_mail — $rej_proc %</p>" >> $tmp/$ST_FILE
echo "<p>$html_spam $spamassassin_mail — $spam_proc %</p>" >> $tmp/$ST_FILE
echo "<p>$html_normal $sent_mail — $sent_proc %</p>" >> $tmp/$ST_FILE
echo "</body></html>" >> $tmp/$ST_FILE
}



function find_mail () {
spamassassin_mail=`cat $log| grep -E ^"$1"|grep "spamd: result: Y"|wc -l`;
tmp_mail=`cat $old_log| grep -E ^"$1"|grep "spamd: result: Y"|wc -l`;
let "spamassassin_mail = spamassassin_mail + tmp_mail"

reject_mail=`cat $log| grep -E ^"$1"|grep "reject"|wc -l`;
tmp_mail=`cat $old_log| grep -E ^"$1"|grep "reject"|wc -l`;
let "reject_mail = reject_mail + tmp_mail"

sent_mail=`cat $log| grep -E ^"$1"|grep "status=sent"|wc -l`;
tmp_mail=`cat $old_log| grep -E ^"$1"|grep "status=sent"|wc -l`;
let "sent_mail = sent_mail + tmp_mail"

let "all_mail = spamassassin_mail + sent_mail + reject_mail"

if [ "$max_mail" -lt "$all_mail" ] 
then max_mail=$all_mail 
fi
}




case "$1" in
day)
#start_date=`LC_ALL=C date --date="yesterday" +%b' '%e`
start_date=`date --date="yesterday" +%b' '%e`

find_mail "$start_date"
#echo "`LC_ALL=C date --date="yesterday" +%m/%d` "$reject_mail $spamassassin_mail $sent_mail> $tmp/mail.dat
echo "`date --date="yesterday" +%m/%d` "$reject_mail $spamassassin_mail $sent_mail> $tmp/mail.dat
create_gnuplot
create_html day

$HTML2PDF $tmp/$ST_FILE "$tmp/statistic-$start_date.pdf"
export EMAIL=$MAILFROM && mutt -e "set content_type=text/html" -a "$tmp/statistic-$start_date.pdf" -s "$SUBJECT" -- $addr < $tmp/$ST_FILE
;;

week)
declare -a mail_count;
j=1
for ((i=7; i>=1; i--)); do
#start_date=`LC_ALL=C date --date="-$i days" +%b' '%e`
start_date=`date --date="-$i days" +%b' '%e`
find_mail "$start_date"

mail_count[$j]=$reject_mail
mail_count[$j+1]=$spamassassin_mail
mail_count[$j+2]=$sent_mail
let "j = j + 3"
done


j=1
if [ -e $tmp/mail.dat ]
then rm $tmp/mail.dat
fi
touch $tmp/mail.dat
for ((i=1; i<=7; i++)); do
let "d = $i -8"
#echo  `LC_ALL=C date --date="$d days" +%m/%d` ${mail_count[$j]} ${mail_count[$j+1]} ${mail_count[$j+2]} >> $tmp/mail.dat
echo  `date --date="$d days" +%m/%d` ${mail_count[$j]} ${mail_count[$j+1]} ${mail_count[$j+2]} >> $tmp/mail.dat
let "j = j + 3"
done

reject_mail=0
spamassassin_mail=0
sent_mail=0


i=1
while [ $i -lt 21 ]
do
let "reject_mail = reject_mail + mail_count[$i]"
let "spamassassin_mail = spamassassin_mail + mail_count[$i+1]"
let "sent_mail = sent_mail + mail_count[$i+2]"
 i=$[$i+3]
done


let "all_mail = spamassassin_mail + sent_mail + reject_mail"

create_gnuplot

create_html week
$HTML2PDF $tmp/$ST_FILE "$tmp/statistic-week.pdf"
export EMAIL=$MAILFROM && mutt -e "set content_type=text/html" -a "$tmp/statistic-week.pdf" -s "$SUBJECT" -- $addr < $tmp/$ST_FILE

;;

*)                                                                                                                                                                                              
echo "Usage: $SCRIPTNAME {day|week}" >&2
exit 1
;;
esac

rm -r $tmp
exit 0
